/* gemskel.h - header file for GEM application skeleton
 * Copyright 1990, Atari Corporation
 * ================================================================
 * 900129 kbad reviewed
 * 890716 kbad Modified to use *event as a quit flag.
 * 890708 kbad Created
 */

#ifndef __AESALT__
#include "../alt/aesalt.h" /* alternate aes bindings, gets aes, stddef */
#endif

#define AES_Version aes_global[0]

/* Window info structure, demonstrated in textwind.c
 */
typedef struct _winfo{
		struct _winfo *next;
		struct _winfo *prev;
		_WORD	id;				/* window handle */
		GRECT	r;				/* window position */
		GRECT	work;			/* working rectangle */
		GRECT	save;			/* saved window position */
		BOOLEAN fulled;			/* window is fulled */
		_WORD	kind;			/* wind_create flags */
		char	*name;			/* window name */
		GRECT	phy;			/* physical coordinates */
		_WORD 	wchar, hchar;	/* dimensions of a "character" */
		GRECT	vir;			/* virtual coordinates */
		void	*x;				/* extended info (app. specific) */
} WINFO;

/* Global variables (defined in gemskel.c unless noted)
 * ================================================================
 */

extern	short _app;
/* application flag defined in startup code. if(!_app) we are a DA
 */

/* VDI stuff
 */
extern _WORD work_out[57];
extern _WORD vhandle;		/* virtual screen workstation handle */
extern _WORD phys_handle;
extern _WORD xres, yres;		/* screen resolution */

/* AES stuff
 */
extern _WORD	gl_hchar,		/* bounds of a character cell */
				gl_wchar,
				gl_hbox,		/* square character bounding box */
				gl_wbox;
extern GRECT	desk;			/* desktop work rectangle */
extern WINFO	*window1;		/* start of linked window list,
								 * declared in textwind.c
								 */

extern _WORD AES_Version;

/* Application specific global functions
 * ================================================================
 */

BOOLEAN	rsrc_init( void );

/* Resource init hook, responsible for loading and initializing
 * resource, setting the address of the menu object (see below), and
 * performing menu_register() if needed for DA.
 * If an error occurs during resource initialization, return FALSE,
 * and the application will exit with an alert. Otherwise return TRUE.
 */

void wind_init( void );

/* Window init hook, called before first evnt_multi.
 */

void evnt_init( void );

/* Hook to initialize evnt_multi parameters.  This is the last thing called
 * before the application goes into the main evnt_multi loop, and it should
 * set the parameters for evnt_multi via evnt_set (see below).
 */

BOOLEAN	evnt_hook( _WORD event, _WORD *msg, MRETS *mrets, _WORD *key, _WORD *nclicks );
/* Hook called immediately after evnt_multi, before the event is handled.
 * Return TRUE to override default event handling, or
 * FALSE to continue with default event handling.
 */

/* Event handling functions
 * =================================================================
 *
 * Mask `*event' as necessary to disable further event processing.
 * The handlers are listed here in the order they are used
 * in the default gemskel.
 * Setting *event to 0 will quit the application.
 */

void do_timer( _WORD *event );
void do_key(_WORD kstate, _WORD key, _WORD *event);
void do_button(MRETS *mrets, _WORD nclicks, _WORD *event);
void do_m1( MRETS *mrets, _WORD *event );
void do_m2( MRETS *mrets, _WORD *event );


/* Message handling functions
 * Mask *event as necessary to disable further event processing
 * (in the default gemskel, message processing is last, though).
 * Setting *event to 0 will quit the application.
 * `msg_hook' is called to handle non-standard AES pipeline messages.
 */
void	do_menu( const _WORD *msg, _WORD *event );
void	do_windows(_WORD *msg, _WORD *event );
void	acc_open( const _WORD *msg );
void	acc_close( const _WORD *msg );
void	msg_hook( const _WORD *msg, _WORD *event );

/* Window drawing functions
 */
void	redraw( const WINFO *w, GRECT *clip );
/* App-specific redraw procedure, called to redraw all or
 * parts of windows.
 */
BOOLEAN	wind_in( int *msg, WINFO *w );
/* Window hook, called before a window message is handled.
 * Return TRUE to override the default message handling,
 * and have do_windows() do_nothing.
 */
void	wind_adjust( const WINFO *w, GRECT *rect );
/* Adjust the coordinates of the window for scrolling redraws.
 * Useful for text windows, which need about a half character
 * width on the left to look nice.
 */
BOOLEAN wind_out( const int *msg, WINFO *w );
/* Window hook called after window message is handled.
 * Its return value is also returned by do_windows() -
 * TRUE to quit the application, FALSE to continue.
 */

/* Termination hooks.
 */
void	wind_exit( void );
/* Window termination hook, responsible for shutting down and deallocating
 * all windows owned by the application.
 */
void	rsrc_exit( void );
/* Resource termination hook, responsible for turning off the menu
 * and deallocating the resource.
 */

/* Global functions defined in gemskel.c
 * ================================================================
 */
void	evnt_set( int mask, int clicks, int bmask, int bstate,
					MOBLK *m1, MOBLK *m2, long time );
/* Sets event parameters for the next iteration of the main evnt_multi.
 * Set unused integral parameters to -1, unused MOBLK pointers to NULL.
 * This function should be called by evnt_init() (see above).
 * WARNING: setting mask to 0 will terminate an application, or send
 *			an accessory into a permanent infinite loop.
 */
void	gem_exit( int code );
/* Exits the application, returning `code' to the parent.
 */

