#include <stddef.h>
#include <string.h>
#include <stdlib.h>
#include <gemx.h>
#include <osbind.h>
#include <country.h>
#include <string.h>

#include "cpxdata.h"

#include "prop_lib.h"

#include "xabubble.h"
#include "xabubble.rsh"

XCPB *xcpb;

_WORD aes_handle;
_WORD gl_wchar;
_WORD gl_hchar;
_WORD gl_wbox;
_WORD gl_hbox;

_WORD vdi_handle;

char xa_mode;

/*
 * Set translatable text fields in resource
 */
static void set_texts(_WORD country)
{
	const _WORD *p;
	OBJECT *tree = rs_trindex[BGMAIN];

	static _WORD const trans_table[2][11] = {
		{
			/* english */
			FRAMETITLE_EN,
			ST_SAVE_EN,
			ST_CANCEL_EN,
			ST_OK_EN,
			ST_WINSTYLE_EN,
                        ST_BUBBLES_EN,
                        ST_DISABLED_EN,
		},
		{
			/* german */
			FRAMETITLE_DE,
			ST_SAVE_DE,
			ST_CANCEL_DE,
			ST_OK_DE,
                        ST_WINSTYLE_DE,
                        ST_BUBBLES_DE,
                        ST_DISABLED_DE,
		}
	};

#define XText(obj,string) tree[obj].ob_spec.tedinfo->te_ptmplt = rs_frstr[string]
#define XString(obj,string) tree[obj].ob_spec.free_string = rs_frstr[string]

	switch (country)
	{
	default:
		/* Default case is USA/UK */
		p = trans_table[0];
		break;
	case COUNTRY_DE:
	case COUNTRY_SG:
		p = trans_table[1];
		break;
	}

	XString(BGMFRAME, *p++);
	XString(BGMSAVE, *p++);
	XString(BGMCANCEL, *p++);
	XString(BGMOK, *p++);
	XString(BGMWINSTYLE, *p++);
        XString(BGMBUBBLES, *p++);
        XString(BGMDISABLED, *p++);
	
#undef XText
#undef XString

}

static void read_config(void)
{
        char buf[10];
        xa_mode = '0';
        short ret = prop_get("xaaes\\xaaes", "xa_bubble", buf, 10, 0);
        if (ret >= 0) /* no error */
        {
            xa_mode = buf[0];
            if (xa_mode != '1' && xa_mode != '2')   /* allows only modes 1 & 2 */
                    xa_mode = '0';
        }
        else Cconws("readconfig error");
}

static void set_config(void)
{
        char buf[2];
        short ret;
        if (xa_mode == '0')
                ret = prop_del("xaaes\\xaaes", "xa_bubble", 0);
        else
        {
                buf[0] = xa_mode;
                buf[1] = '\0';
                ret = prop_put("xaaes\\xaaes", "xa_bubble", buf, 0);
        }
        if (ret < 0) form_alert(1, "[1][Unable to save settings.][OK]");
}

static void redraw_obj(OBJECT *tree, _WORD obj)
{
	GRECT *clip_p;
	GRECT clip;
	GRECT xywh;
	GRECT full;

	wind_get_grect(0, WF_WORKXYWH, &full);

	objc_offset(tree, ROOT, &xywh.g_x, &xywh.g_y);
	xywh.g_w = tree->ob_width;
	xywh.g_h = tree->ob_height;

	clip_p = cpxGetFirstRect(xcpb, &xywh);

	while (clip_p)
	{
		clip = *clip_p;

		if (rc_intersect(&full, &clip))
			objc_draw_grect(tree, obj, MAX_DEPTH, &clip);

		clip_p = cpxGetNextRect(xcpb);
	}
}

static _WORD handle_button(_WORD button, _WORD *msg)
{
	_WORD quit = FALSE;
	OBJECT *cpx_tree = rs_trindex[BGMAIN];

	/*
	 *  button: -1      : msg[] enthlt ein auszuwertende Nachricht
	 *            sonst : Nummer des angeklickten Objekts
	 *                        (gesetzes Bit 15: Doppelklick auf Objekt)
	 */
	if (button != -1)
		button &= 0x7fff;				/* Doppelklick interessiert nicht */

	switch (button)
	{
	case BGMOK:
            /* read value */
            if (cpx_tree[BGMBUBBLES].ob_state & OS_SELECTED)
                xa_mode = '2';
            else if  (cpx_tree[BGMWINSTYLE].ob_state & OS_SELECTED)
                xa_mode = '1';
            else if (cpx_tree[BGMDISABLED].ob_state & OS_SELECTED)
                xa_mode = '0';

            cpx_tree[BGMOK].ob_state &= ~OS_SELECTED;
		set_config();
		quit = TRUE;
		break;
	case BGMCANCEL:
		cpx_tree[BGMCANCEL].ob_state &= ~OS_SELECTED;
		quit = TRUE;
		break;
	case BGMSAVE:
            /* read value */
                if (cpx_tree[BGMBUBBLES].ob_state & OS_SELECTED)
                    xa_mode = '2';
                else if  (cpx_tree[BGMWINSTYLE].ob_state & OS_SELECTED)
                    xa_mode = '1';
                else if (cpx_tree[BGMDISABLED].ob_state & OS_SELECTED)
                    xa_mode = '0';
		set_config();
		cpx_tree[BGMSAVE].ob_state &= ~OS_SELECTED;
		redraw_obj(cpx_tree, BGMSAVE);
		break;
	case -1:
		switch (msg[0])
		{
		case AC_CLOSE:
			quit = TRUE;
			break;
		case WM_CLOSED:
			set_config();
			quit = TRUE;
			break;
		}
		break;
	}

	return quit;
}

static _WORD __CDECL cpx_call(GRECT *rect)
{
	OBJECT *cpx_tree = rs_trindex[BGMAIN];
	_WORD msg[8];
	_WORD button;
	_WORD quit = FALSE;

	aes_global[0] = 0; /* reset AES version */
	aes_global[2] = 0; /* reset application ID */

	aes_handle = graf_handle(&gl_wchar, &gl_hchar, &gl_wbox, &gl_hbox);	
        
	cpx_tree->ob_x = rect->g_x;
	cpx_tree->ob_y = rect->g_y;

	read_config();

        /* update form */
        cpx_tree[BGMBUBBLES].ob_state &= ~OS_SELECTED;
        cpx_tree[BGMWINSTYLE].ob_state &= ~OS_SELECTED;
        cpx_tree[BGMDISABLED].ob_state &= ~OS_SELECTED;
        if (xa_mode == '0')
                cpx_tree[BGMDISABLED].ob_state |= OS_SELECTED;
        else if (xa_mode == '2')
                cpx_tree[BGMBUBBLES].ob_state |= OS_SELECTED;
        else if (xa_mode == '1')
                cpx_tree[BGMWINSTYLE].ob_state |= OS_SELECTED; 
                
	redraw_obj(cpx_tree, ROOT);
	do
	{
		/* BubbleHelp */
		button = cpxXform_do(xcpb, cpx_tree, 0, msg);
		quit = handle_button(button, msg);
	} while (!quit);

	return 0;
}

CPXINFO cpxinfo = {
	cpx_call,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0,
	0
};

CPXINFO *__CDECL cpx_init(XCPB *Xcpb)
{
	xcpb = Xcpb;

	set_texts(xcpb->Country_Code);

	if (xcpb->booting)
	{
		return (CPXINFO *) 1;
	}

	if (xcpb->SkipRshFix == 0)
	{
		_WORD i;
		OBJECT *cpx_tree = rs_trindex[BGMAIN];

		for (i = 0; i < NUM_OBS; i++)
			cpxrsh_obfix(xcpb, cpx_tree, i);
	}

	return &cpxinfo;
}
